<?php

namespace Drupal\role_toggle\AccountDecorator;

use Drupal\Core\Session\AccountInterface;

abstract class AccountDecoratorBase implements AccountInterface, DecoratedAccountInterface {

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $decorated;

  /**
   * AccountProxyDecoratorBase constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $decorated
   */
  public function __construct(AccountInterface $decorated) {
    $this->decorated = $decorated;
  }

  /**
   * @return \Drupal\Core\Session\AccountInterface
   */
  public function getDecorated() {
    return $this->decorated;
  }

  /**
   * @inheritDoc
   */
  public function id() {
    return $this->getDecorated()->id();
  }

  /**
   * @inheritDoc
   */
  public function getRoles($exclude_locked_roles = FALSE) {
    return $this->getDecorated()->getRoles($exclude_locked_roles);
  }

  /**
   * @inheritDoc
   */
  public function hasPermission($permission) {
    return $this->getDecorated()->hasPermission($permission);
  }

  /**
   * @inheritDoc
   */
  public function isAuthenticated() {
    return $this->getDecorated()->isAuthenticated();
  }

  /**
   * @inheritDoc
   */
  public function isAnonymous() {
    return $this->getDecorated()->isAnonymous();
  }

  /**
   * @inheritDoc
   */
  public function getPreferredLangcode($fallback_to_default = TRUE) {
    return $this->getDecorated()->getPreferredLangcode($fallback_to_default);
  }

  /**
   * @inheritDoc
   */
  public function getPreferredAdminLangcode($fallback_to_default = TRUE) {
    return $this->getDecorated()->getPreferredAdminLangcode($fallback_to_default);
  }

  /**
   * @inheritDoc
   */
  public function getUsername() {
    return $this->getDecorated()->getUsername();
  }

  /**
   * @inheritDoc
   */
  public function getAccountName() {
    return $this->getDecorated()->getAccountName();
  }

  /**
   * @inheritDoc
   */
  public function getDisplayName() {
    return $this->getDecorated()->getDisplayName();
  }

  /**
   * @inheritDoc
   */
  public function getEmail() {
    return $this->getDecorated()->getEmail();
  }

  /**
   * @inheritDoc
   */
  public function getTimeZone() {
    return $this->getDecorated()->getTimeZone();
  }

  /**
   * @inheritDoc
   */
  public function getLastAccessedTime() {
    return $this->getDecorated()->getLastAccessedTime();
  }

}
