<?php

namespace Drupal\role_toggle\AccountDecorator;

use Drupal\Core\Session\AccountInterface;
use Drupal\user\RoleStorageInterface;

class RoleToggleAccountDecorator extends AccountDecoratorBase {

  /**
   * @var \Drupal\user\RoleStorageInterface
   */
  private $roleStorage;

  /**
   * @var string[];
   */
  protected $roles;

  /**
   * @var string[];
   */
  protected $extraPermissions;

  /**
   * RoleToggleAccountDecorator constructor.
   *
   * @param \Drupal\Core\Session\AccountInterface $decorated
   * @param \Drupal\user\RoleStorageInterface $roleStorage
   * @param string[] $roles
   * @param string[] $extraPermissions
   */
  public function __construct(AccountInterface $decorated, RoleStorageInterface $roleStorage, $roles, $extraPermissions) {
    parent::__construct($decorated);
    $this->roleStorage = $roleStorage;
    $this->roles = $roles;
    $this->extraPermissions = $extraPermissions;
  }

  /**
   * {@inheritDoc}
   */
  public function getRoles($exclude_locked_roles = FALSE) {
    $roles = $this->roles;
    if ($exclude_locked_roles) {
      $roles = array_values(array_diff($roles, [AccountInterface::ANONYMOUS_ROLE, AccountInterface::AUTHENTICATED_ROLE]));
    }
    return $roles;
  }

  /**
   * {@inheritDoc}
   */
  public function hasPermission($permission) {
    if (in_array($permission, $this->extraPermissions)) {
      return TRUE;
    }
    return $this->getRoleStorage()->isPermissionInRoles($permission, $this->getRoles());
  }

  /**
   * @return \Drupal\user\RoleStorageInterface
   */
  public function getRoleStorage() {
    return $this->roleStorage;
  }

}
