<?php

namespace Drupal\role_toggle\CacheContext;

use Drupal\Core\Cache\Context\CacheContextInterface;

class CacheContextDecoratorBase implements CacheContextInterface {

  /**
   * The decorated object.
   * 
   * @var \Drupal\Core\Cache\Context\CacheContextInterface
   */
  protected $decorated;

  /**
   * CacheContextDecoratorBase constructor.
   *
   * @param \Drupal\Core\Cache\Context\CacheContextInterface $decorated
   */
  public function __construct(CacheContextInterface $decorated) {
    $this->decorated = $decorated;
  }

  /**
   * Hack for getLabel, which is declared static but called dynamically.
   *
   * @see \Drupal\Core\Cache\Context\CacheContextsManager::getLabels
   *
   * @param $name
   * @param $args
   *
   * @return string
   */
  public function __call($name, $args) {
    if ($name === 'getLabel') {
      return $this->decorated->getLabel();
    }
    else {
      throw new \UnexpectedValueException();
    }
  }

  /**
   * { @inheritDoc }
   */
  public static function getLabel() {
    // TODO: Implement getLabel() method.
  }

  /**
   * { @inheritDoc }
   */
  public function getContext() {
    return $this->decorated->getContext();
  }

  /**
   * { @inheritDoc }
   */
  public function getCacheableMetadata() {
    return $this->decorated->getCacheableMetadata();
  }

}
