<?php

namespace Drupal\role_toggle;

use Drupal\Core\Entity\EntityInterface;
use Drupal\user\UserInterface;

class EntityHooks {

  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * RoleToggleToolbar constructor.
   *
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   */
  public function __construct(RoleToggleState $roleToggleState) {
    $this->roleToggleState = $roleToggleState;
  }

  /**
   * Implements hook_entity_load().
   *
   * When roles are toggled, set them on the user.
   *
   * @param \Drupal\Core\Entity\EntityInterface[] $entities
   *   The entities.
   * @param string $entity_type_id
   *   The entity type id.
   */
  public function hookEntityLoad(array $entities, $entity_type_id) {
    if ($entity_type_id === 'user') {
      /** @var \Drupal\user\UserInterface[] $entities */
      if ($this->roleToggleState->rolesAreToggled()) {
        $currentUserId = $this->roleToggleState->getOriginalAccountId();
        if (isset($entities[$currentUserId])) {
          $user = $entities[$currentUserId];
          $user->set('roles', $this->roleToggleState->getToggledRolesBag()->idsWithoutLocked());
        }
      }
    }
  }

  /**
   * Implements hook_entity_presave().
   *
   * When roles are toggled, restore them on the user.
   * This effectively means, role changed are prohibited when toggled.
   */
  public function hookEntityPresave(EntityInterface $entity) {
    if ($entity instanceof UserInterface) {
      $user = $entity;
      if (
        $this->roleToggleState->rolesAreToggled() &&
        $user->id() &&
        $user->id() === $this->roleToggleState->getOriginalAccountId()
      ) {
        $user->set('roles', $this->roleToggleState->getOriginalRolesBag()->idsWithoutLocked());
      }
    }
  }

}
