<?php

namespace Drupal\role_toggle\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\role_toggle\RoleToggleUrlManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

class RoleToggleEventSubscriber implements EventSubscriberInterface {

  /**
   * @var \Drupal\role_toggle\RoleToggleUrlManager
   */
  private $roleToggleUrlManager;

  /**
   * RoleToggleEventSubscriber constructor.
   *
   * @param \Drupal\role_toggle\RoleToggleUrlManager $roleToggleUrlManager
   */
  public function __construct(RoleToggleUrlManager $roleToggleUrlManager) {
    $this->roleToggleUrlManager = $roleToggleUrlManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => 'filterResponse',
    ];
  }

  /**
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   */
  public function filterResponse(FilterResponseEvent $event) {
    $response = $event->getResponse();
    $queryKey = $this->roleToggleUrlManager->queryKey();
    if (
      $response instanceof RedirectResponse
    ) {
      $parsed = UrlHelper::parse($response->getTargetUrl());
      $query = $event->getRequest()->query;
      if (!isset($parsed['query'][$queryKey]) && $query->has($queryKey)) {
        $parsed['query'][$queryKey] = $query->get($queryKey);
      }
      elseif (($parsed['query'][$queryKey] ?? '') === 'empty-redirect') {
        unset($parsed['query'][$queryKey]);
      }
      $url = $parsed['path'];
      if ($parsed['query']) {
        $url .= '?' . UrlHelper::buildQuery($parsed['query']);
      }
      if (strlen($parsed['fragment'])) {
        $url .= '#' . $parsed['fragment'];
      }
      $response->setTargetUrl($url);
    }
  }

}
