<?php

namespace Drupal\role_toggle\PathProcessor;

use Drupal\Core\PathProcessor\OutboundPathProcessorInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\role_toggle\RoleToggleState;
use Drupal\role_toggle\RoleToggleUrlManager;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

class RoleTogglePathProcessor implements OutboundPathProcessorInterface {

  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * @var \Drupal\role_toggle\RoleToggleUrlManager
   */
  private $roleToggleUrlManager;

  /**
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  private $requestStack;

  /**
   * RoleTogglePathProcessor constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   * @param \Drupal\role_toggle\RoleToggleUrlManager $roleToggleUrlManager
   */
  public function __construct(RequestStack $requestStack,  $roleToggleState, RoleToggleUrlManager $roleToggleUrlManager) {
    $this->requestStack = $requestStack;
    $this->roleToggleUrlManager = $roleToggleUrlManager;
    $this->roleToggleState = $roleToggleState;
  }

  /**
   * {@inheritDoc}
   *
   * We copy over our query value from the master request, valid or not.
   * We tried getting the value from the state, but this seems not reliable.
   */
  public function processOutbound($path, &$options = [], Request $request = NULL, BubbleableMetadata $bubbleable_metadata = NULL) {
    if ($bubbleable_metadata) {
      $bubbleable_metadata->addCacheContexts($this->roleToggleUrlManager->getCacheContexts());
    }
    $masterRequest = $this->requestStack->getMasterRequest();
    $queryKey = $this->roleToggleUrlManager->queryKey();
    // Do not change any role switch links.
    if (!isset($options['query'][$queryKey])) {
      if ($masterRequest && $masterRequest->query->has($queryKey)) {
        $queryValue = $masterRequest->query->get($queryKey);
        $options['query'][$queryKey] = $queryValue;
      }
      else {
        unset($options['query'][$queryKey]);
      }
    }
    // We need this magic value so we can have an empty role switch link.
    elseif ($options['query'][$queryKey] === 'empty-link') {
      unset($options['query'][$queryKey]);
    }
    return $path;
  }

}
