<?php

namespace Drupal\role_toggle;

use Drupal\Core\Form\FormStateInterface;
use Drupal\user\RoleInterface;

class RoleExcludedService {

  /**
   * @param $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function formUserRoleFormAlter(&$form, FormStateInterface $form_state) {
    /** @var \Drupal\Core\Entity\EntityFormInterface $formObject */
    $formObject = $form_state->getFormObject();
    // See https://www.drupal.org/node/2818877.
    /** @var \Drupal\field\FieldConfigInterface|\Drupal\Core\Field\FieldConfigInterface $fieldConfig */
    $fieldConfig = $formObject->getEntity();

    $form['third_party_settings']['#tree'] = TRUE;

    $form['third_party_settings']['role_toggle']['#type'] = 'fieldset';
    $form['third_party_settings']['role_toggle']['#title'] = t('Role Toggle');

    $form['third_party_settings']['role_toggle']['excluded'] = [
      '#type' => 'checkbox',
      '#title' => t('Excluded'),
      '#description' => t('This role is excluded from toggling via <em>Toggle roles</em> permission. It can be toggled via <em>Toggle all roles</em> permission though.'),
      '#default_value' => $fieldConfig->getThirdPartySetting('role_toggle', 'excluded'),
    ];
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   *
   * @return bool
   */
  public function isExcludedRole(RoleInterface $role) {
    return $role->getThirdPartySetting('role_toggle', 'excluded');
  }

  /**
   * @param \Drupal\user\RoleInterface $role
   *
   * @return bool
   */
  public function isIncludedRole(RoleInterface $role) {
    return !$this->isExcludedRole($role);
  }


  /**
   * @param \Drupal\user\RoleInterface $role
   *
   * @return string
   */
  public function cssClass(RoleInterface $role) {
    return $this->isExcludedRole($role) ? 'role-toggle-excluded' : 'role-toggle-included';
  }

}
