<?php

namespace Drupal\role_toggle;

use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Request;

/**
 * Translates enabled roles vs. querystring.
 */
class RoleToggleState {

  /**
   * @var int
   */
  protected $originalAccountId;

  /**
   * @var \Drupal\role_toggle\RolesBag
   */
  protected $originalRolesBag;

  /**
   * @var \Drupal\role_toggle\RolesBag
   */
  protected $toggledRolesBag;

  /**
   * @return int
   */
  public function getOriginalAccountId() {
    return $this->originalAccountId;
  }

  /**
   * @return \Drupal\role_toggle\RolesBag
   */
  public function getOriginalRolesBag() {
    return $this->originalRolesBag;
  }

  /**
   * @return \Drupal\role_toggle\RolesBag
   */
  public function getToggledRolesBag() {
    return $this->toggledRolesBag;
  }

  /**
   * @return bool
   */
  public function rolesAreToggled() {
    return $this->initialized() &&
      $this->originalRolesBag->idsWithLocked()
      !== $this->toggledRolesBag->idsWithLocked();
  }

  public function init(AccountInterface $account = NULL) {
    if ($account) {
      $this->originalAccountId = $account->id();
      $this->originalRolesBag = RolesBag::fromAccount($account);
    }
    else {
      $this->originalAccountId = User::getAnonymousUser()->id();
      $this->originalRolesBag = RolesBag::anonymous();
    }
    $this->toggledRolesBag = $this->originalRolesBag;
  }

  public function setAccount(AccountInterface $account) {
    $this->toggledRolesBag = RolesBag::fromAccount($account);
  }

  /**
   * @return bool
   */
  public function initialized() {
    return isset($this->originalAccountId);
  }

}
