<?php

namespace Drupal\role_toggle;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Url;

class RoleToggleToolbar {

  /**
   * The role toggle state.
   *
   * @var \Drupal\role_toggle\RoleToggleState
   */
  private $roleToggleState;

  /**
   * @var \Drupal\role_toggle\RoleToggleUrlManager
   */
  private $roleToggleUrlManager;

  /**
   * @var \Drupal\role_toggle\RoleExcludedService
   */
  private $roleExcludedService;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  private $currentUser;

  /**
   * RoleTogglePathProcessor constructor.
   *
   * @param \Drupal\role_toggle\RoleToggleState $roleToggleState
   * @param \Drupal\role_toggle\RoleToggleUrlManager $roleToggleUrlManager
   * @param \Drupal\role_toggle\RoleExcludedService $roleExcludedService
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   */
  public function __construct(RoleToggleState $roleToggleState, RoleToggleUrlManager $roleToggleUrlManager, RoleExcludedService $roleExcludedService, AccountProxyInterface $currentUser) {
    $this->roleToggleState = $roleToggleState;
    $this->roleToggleUrlManager = $roleToggleUrlManager;
    $this->currentUser = $currentUser;
    $this->roleExcludedService = $roleExcludedService;
  }

  public function build() {
    $items['role_toggle'] = [
      '#cache' => [
        'contexts' => ['user.permissions'],
      ],
    ];

    if ($this->roleToggleUrlManager->toggleableRolesBag()) {
      $title = t('@label: @roles', [
        '@label' => t('Roles'),
        '@roles' => new FormattableMarkup($this->rolesIndicator(), []),
      ]);
      $items['role_toggle'] += [
        '#type' => 'toolbar_item',
        '#weight' => 999,
        'tab' => [
          '#type' => 'link',
          '#title' => $title,
          '#url' => Url::fromRoute('role_toggle.form'),
          '#attributes' => [
            'title' => t('Role toggle'),
            'class' => ['toolbar-icon', 'toolbar-icon-role-toggle'],
          ],
        ],
        'tray' => [
          'role_toggle_form' => [
            '#lazy_builder' => [RoleToggleToolbar::class . '::lazyBuilder', [
              implode('+', $this->roleToggleState->getToggledRolesBag()->idsWithoutLocked())
            ]],
            '#create_placeholder' => TRUE,
          ],
        ],
        '#attached' => [
          'library' => 'role_toggle/toolbar',
        ],
      ];
    }

    return $items;
  }

  public static function lazyBuilder($implodedRoleIds) {
    /** @var \Drupal\Core\Form\FormBuilderInterface $form_builder */
    $form_builder = \Drupal::service('form_builder');
    $roleIds = explode('+', $implodedRoleIds);
    $form = $form_builder->getForm(RoleToggleForm::class, $roleIds);
    return $form;
  }

  private function rolesIndicator() {
    if ($this->roleToggleState->getToggledRolesBag()->hasAuthenticated()) {
      $originalRoles = $this->roleToggleState->getOriginalRolesBag()->rolesWithoutLocked();
      $toggledRoleIds = $this->roleToggleState->getToggledRolesBag()->idsWithoutLocked();
      $chars = [];
      foreach ($originalRoles as $roleId => $role) {
        $active = in_array($roleId, $toggledRoleIds);
        $label = $role->label();
        $char = substr($label, 0, 1);
        $casedChar = $active ? mb_strtoupper($char) : mb_strtolower($char);
        $class = $this->roleExcludedService->cssClass($role);
        $chars[] = "<span class=\"$class\">$casedChar</span>";
      }
      return implode($chars);
    }
    else {
      return t('Anonymous');
    }
  }

}
